# master setup 실행한 다음 자동으로 서버를 system service로 등록하기
import logging

import click

from master import utils
from master.cmd.setup import setup as master_setup
from master.cmd.setup import setup_calico as master_setup_calico
from master.cmd.setup import setup_system_service as master_setup_system_service
from master.cmd.serve import serve as master_serve
from master.cmd.serve import PORT

logging.basicConfig(
    format='%(asctime)s %(levelname)-8s %(message)s',
    level=logging.INFO,
    datefmt='%Y-%m-%d %H:%M:%S')


@click.group()
def cli():
    pass


@cli.command()
@click.option("--nic", help="(Optional) Network interface card to communicate with workers. Default to first NIC (e.g. eth0)", type=click.STRING, required=False)
@click.option("--pod-cidr", help="(Optional) Pod CIDR for kubernetes. Default to 192.168.0.0/16", type=click.STRING, required=False)
@click.option("--calico-only", help="(Optional) Install calico only. Can be used with --pod-cidr", is_flag=True, required=False)
def setup(nic, pod_cidr, calico_only):
    if calico_only:
        master_setup_calico(pod_cidr=pod_cidr)
    if not utils.check_kubeadm_initialized():
        if master_setup():
            master_setup_calico(pod_cidr=pod_cidr)
            master_setup_system_service()

    logging.info(f"""\n
    Run below command in all worker servers.
    > agent-worker join --master-endpoint={utils.get_api_server_address(nic)}:{PORT}
    """)


@cli.command()
def serve():
    master_serve()


if __name__ == "__main__":
    cli()


import os
import random
import string
import subprocess
from functools import lru_cache
import time
import urllib3

import netifaces as ni
import requests

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def get_default_iface():
    default_iface = [iface for iface in os.listdir('/sys/class/net/') if iface != 'lo' and iface != 'docker0'][0]
    return default_iface


@lru_cache()
def get_join_token(ttl_hash=None):
    if check_kubeadm_initialized():
        return str(subprocess.check_output(['kubeadm', 'token', 'create']).decode("utf-8").strip())
    else:
        return f'{get_random_string(6)}.{get_random_string(16)}'


def get_ttl_hash(seconds=1800):
    return round(time.time() / seconds)


# 바꾸지 않아도 문제는 없으니 일단 이대로 놔둔다.
def get_compatible_versions():
    kube_version = "1.20.8-00"
    cni_version = "0.8.7-00"
    containerd_version = "1.5.2-0ubuntu1~18.04.2"

    return kube_version, cni_version, containerd_version


# default iface의 1번째 IP만을 반환한다. NIC에 IP가 여러개 붙어있는 경우는 고려하지 않음.
def get_api_server_address(nic):
    if nic:
        return ni.ifaddresses(nic)[ni.AF_INET][0]['addr']
    else:
        return ni.ifaddresses(get_default_iface())[ni.AF_INET][0]['addr']


def get_random_string(length):
    return ''.join(random.choices(string.ascii_lowercase + string.digits, k=length))


def check_kubeadm_initialized():
    try:
        response = requests.get('https://localhost:6443', verify=False)
        if response.status_code == 403: # If api server is running, 403 should be returned
            return True
    except requests.exceptions.ConnectionError:
        return False
